#include "boite.h"

Boite::~Boite()
{
   // Ne fait rien, l'objet est dtruit par la classe de base
}

void Boite::Initialiser (NewtonWorld * nWorld, const CVector & taille,
                         const CVector & position, GLboolean mobile, GLfloat masse)
{
   // On initialise le vecteur de dimensions
   m_taille.x = taille.x;
   m_taille.y = taille.y;
   m_taille.z = taille.z;

   // On dfinit la masse de l'objet
   m_masse = masse;

   matrix matrice; // On cr une matrice

   // On initialise la matrice, qu'on dfinit comme matrice identit
   for (int x = 0 ; x < 4 ; ++x)
      for (int y = 0 ; y < 4 ; y++)
      {
         if (x == y)
            matrice.matrice [x][y] = 1.0f;
         else
            matrice.matrice [x][y] = 0.0f;
      }

   // On dfinit la matrice de manire  ce que l'objet soit plac aux positions
   // spcifies en utilisant la dernire colonne de la matrice
   matrice.matrice [3][0] = position.x;
   matrice.matrice [3][1] = position.y;
   matrice.matrice [3][2] = position.z;

   // On initialise la collision box
   NewtonCollision * collision = NULL;

   // On cr la collision box aux dimensions de l'objet
   collision = NewtonCreateBox (nWorld, m_taille.x, m_taille.y, m_taille.z, NULL);

   // On initialise le corps avec la collision box
   m_pBody = NewtonCreateBody (nWorld, collision);

   if (m_pBody == NULL)
      std::cerr << "Impossible d'initialiser le corps.";

   // On dtruit la collision box, on n'en a plus besoin
   NewtonReleaseCollision (nWorld, collision);

   // Enfin, on assigne notre matrix (qui reprsente donc sa position dans l'espace)
   //  notre corps grce  la fonction NewtonBodySetMatrix
   NewtonBodySetMatrix (m_pBody, &matrice.matrice [0][0]);

   // On initialise  prsent les proprits physiques de l'objet. Toutefois, donner
   //  un objet qui ne bougera pas une masse, lui associer un callback,... n'a aucun
   // intret, on vrifie donc si l'objet sera mobile ou immobile
   if (mobile == true)
   {
      // On calcul l'inertie du corps, en passant par une petite formule
      CVector inertie;

      inertie.x = 0.7f * m_masse * (m_taille.y * m_taille.y + m_taille.z * m_taille.z) / 12;
      inertie.y = 0.7f * m_masse * (m_taille.x * m_taille.x + m_taille.z * m_taille.z) / 12;
      inertie.z = 0.7f * m_masse * (m_taille.x * m_taille.x + m_taille.y * m_taille.y) / 12;

      // On dfinit ensuite la masse et l'inertie pour ce corps
      NewtonBodySetMassMatrix (m_pBody, m_masse, inertie.x, inertie.y, inertie.z);

      // On rgle enfin le Callback, qui sera ncessaire pour que le corps bouge
      NewtonBodySetForceAndTorqueCallback (m_pBody, ApplyForceAndTorqueCallback);
   }

   m_taille.x *= 0.5f;
   m_taille.y *= 0.5f;
   m_taille.z *= 0.5f;
}

void Boite::SetColor (const CVector & couleur)
{
   m_couleur.x = couleur.x / 255;
   m_couleur.y = couleur.y / 255;
   m_couleur.z = couleur.z / 255;
}

void Boite::Dessiner ()
{
   // Toutes les modifications effectues dans le Callback modifient la matrice de
   // l'objet, ce qui permet  l'objet de "bouger"
   matrix matrice;

   NewtonBodyGetMatrix (m_pBody, &matrice.matrice [0][0]);

   glPushMatrix (); // On sauvegarde la matrice actuelle
      glMultMatrixf (&matrice.matrice [0][0]); // On multiplie la matrice actuelle
                                               // par la matrice du corps, ainsi
                                               // le corps sera dessin au bon endroit
      glColorMaterial (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
      glColor3f (m_couleur.x, m_couleur.y, m_couleur.z);
      glColorMaterial (GL_FRONT_AND_BACK, GL_SPECULAR);
      glColor3f (1.0f, 1.0f, 1.0f);
      glColorMaterial (GL_FRONT_AND_BACK, GL_EMISSION);
      glColor3f (0.0f, 0.0f, 0.0f);
      glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 128);
      glBegin(GL_QUADS);
         // Devant
         glNormal3f (0.0f, 0.0f, 1.0f);
         glVertex3f (-m_taille.x, -m_taille.y, m_taille.z);
         glVertex3f (m_taille.x, -m_taille.y, m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, m_taille.z);
         glVertex3f (-m_taille.x, m_taille.y, m_taille.z);

         // Derrire
         glNormal3f (0.0f, 0.0f, -1.0f);
         glVertex3f (-m_taille.x, -m_taille.y, -m_taille.z);
         glVertex3f (-m_taille.x, m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, -m_taille.y, -m_taille.z);

         // Haut
         glNormal3f (0.0f, 1.0f, 0.0f);
         glVertex3f (-m_taille.x, m_taille.y, -m_taille.z);
         glVertex3f (-m_taille.x, m_taille.y,  m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, -m_taille.z);

         // Bas
         glNormal3f (0.0f, -1.0f, 0.0f);
         glVertex3f (-m_taille.x, -m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, -m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, -m_taille.y, m_taille.z);
         glVertex3f (-m_taille.x, -m_taille.y, m_taille.z);

         // Droite
         glNormal3f (1.0f, 0.0f, 0.0f);
         glVertex3f (m_taille.x, -m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, -m_taille.z);
         glVertex3f (m_taille.x, m_taille.y, m_taille.z);
         glVertex3f (m_taille.x, -m_taille.y, m_taille.z);

         // Gauche
         glNormal3f (-1.0f, 0.0f, 0.0f);
         glVertex3f (-m_taille.x, -m_taille.y, -m_taille.z);
         glVertex3f (-m_taille.x, -m_taille.y, m_taille.z);
         glVertex3f (-m_taille.x, m_taille.y, m_taille.z);
         glVertex3f (-m_taille.x, m_taille.y, -m_taille.z);
      glEnd ();
   glPopMatrix (); // On rtablit la matrice
}

